@@ -45,7 +45,7 @@
 #include <linux/hdreg.h>
 #include <linux/fs.h>		/* for BLKRRPART */
 
-#define VERSION "0.7 ALPHA"
+#define VERSION "0.8 BETA"
 
 #define DEFAULT_DEVICE "/dev/hda"
 #define ALTERNATE_DEVICE "/dev/sda"
@@ -127,6 +127,9 @@
 #define IS_PRIMARY(p) ((p) >= 0 && (p) < 4)
 #define IS_LOGICAL(p) ((p) > 3)
 
+#define round_int(d) ((double)((int)(d+0.5)))
+#define ceiling(d) ((double)(((d) != (int)(d)) ? (int)(d+1.0) : (int)(d)))
+
 #define set_hsc(h,s,c,sector) \
 { \
       s = sector % sectors + 1;	\
@@ -357,6 +360,7 @@ void fatal(char *s)
     signal(SIGINT, old_SIGINT);
     signal(SIGTERM, old_SIGTERM);
     mvcur(0, COLS-1, LINES-1, 0);
+    nl();
     endwin();
     fdexit(1);
 }
@@ -714,13 +718,13 @@ void new_part(int i)
     else
 	print_warning("!!! Internal error !!!");
 
-    sprintf(def, "%.2f", ceil(num_sects/20.48)/100);
+    sprintf(def, "%.2f", ceiling(num_sects/20.48)/100);
     mvaddstr(COMMAND_LINE_Y, COMMAND_LINE_X, "Size (in MB): ");
     if ((len = get_string(response, LINE_LENGTH, def)) <= 0 &&
 	len != GS_DEFAULT)
 	return;
     else if (len > 0) {
-#define num_cyls(bytes) (rint(bytes/SECTOR_SIZE/(sectors*heads)))
+#define num_cyls(bytes) (round_int(bytes/SECTOR_SIZE/(sectors*heads)))
 	for (j = 0;
 	     j < len-1 && (isdigit(response[j]) || response[j] == '.');
 	     j++);
@@ -729,9 +733,9 @@ void new_part(int i)
 	} else if (toupper(response[j]) == 'M') {
 	    num_sects = num_cyls(atof(response)*1024*1024)*sectors*heads;
 	} else if (toupper(response[j]) == 'C') {
-	    num_sects = rint(atof(response))*sectors*heads;
+	    num_sects = round_int(atof(response))*sectors*heads;
 	} else if (toupper(response[j]) == 'S') {
-	    num_sects = rint(atof(response));
+	    num_sects = round_int(atof(response));
 	} else {
 	    num_sects = num_cyls(atof(response)*1024*1024)*sectors*heads;
 	}
@@ -1664,10 +1668,10 @@ void draw_partition(int i)
     else if (display_units == CYLINDERS)
 	mvprintw(y, SIZE_START, "%9d", size/(sectors*heads));
     else
-	mvprintw(y, SIZE_START, "%9.2f", ceil(size/20.48)/100);
-    if (((size/(sectors*heads)) != ceil(size/(sectors*(float)heads))) ||
+	mvprintw(y, SIZE_START, "%9.2f", ceiling(size/20.48)/100);
+    if (((size/(sectors*heads)) != ceiling(size/(sectors*(float)heads))) ||
 	((p_info[i].first_sector/(sectors*heads)) !=
-	 ceil(p_info[i].first_sector/(sectors*heads))))
+	 ceiling(p_info[i].first_sector/(sectors*heads))))
 	mvprintw(y, COLUMNS-1, "*");
 }
 
@@ -1783,6 +1787,7 @@ void die(int dummy)
     signal(SIGINT, old_SIGINT);
     signal(SIGTERM, old_SIGTERM);
     mvcur(0, COLS-1, LINES-1, 0);
+    nl();
     endwin();
     fdexit(0);
 }
@@ -1947,6 +1952,7 @@ void do_curses_fdisk(void)
     signal(SIGINT, old_SIGINT);
     signal(SIGTERM, old_SIGTERM);
     mvcur(0, COLS-1, LINES-1, 0);
+    nl();
     endwin();
     fdexit(0);
 }
