@@ -9,6 +9,7 @@
 
 int reboot(int magic, int magictoo, int flag);
 
+int
 main(int argc, char *argv[]) {
 
 	if(geteuid()) {
@@ -30,6 +31,7 @@ main(int argc, char *argv[]) {
 		fprintf(stderr, "Usage: ctrlaltdel hard|soft\n");
 		exit(1);
 	}
+	exit(0);
 }
 
 
