@@ -28,7 +28,9 @@
  *    1) Added support for DOS, OS/2, ... compatibility.  We should be able
  *       use this fdisk to partition our drives for other operating systems.
  *    2) Added a print the raw data in the partition table command.
- *
+ * Modified, Wed Jun 22 21:05:30 1994, faith@cs.unc.edu:
+ *    Added/changed a few partition type names to conform to cfdisk.
+ *    (suggested by Sujal, smpatel@wam.umd.edu)
  */
 
 
@@ -54,7 +56,7 @@
 			})
 
 
-#define VERSION	"1.5a"
+#define VERSION	"1.5b"
 
 #define DEFAULT_DEVICE	"/dev/hda"
 #define ALTERNATE_DEVICE "/dev/sda"
@@ -135,7 +137,7 @@ struct systypes {
 		{7, "OS/2 HPFS"},		/* or QNX? */
 		{8, "AIX"},
 		{9, "AIX bootable"},
-		{10, "OPUS"},
+		{10, "OS/2 Boot Manager"},
 		{0x40, "Venix 80286"},
 		{0x51, "Novell?"},
 		{0x52, "Microport"},		/* or CPM? */
@@ -150,6 +152,7 @@ struct systypes {
 
 		{0x93, "Amoeba"},
 		{0x94, "Amoeba BBT"},		/* (bad block table) */
+		{0xa5, "BSD/386"},
 		{0xb7, "BSDI fs"},
 		{0xb8, "BSDI swap"},
 		{0xc7, "Syrinx"},
