@@ -1,12 +1,28 @@
 /*
  * A mount(8) for Linux 0.99.
  * mount.c,v 1.1.1.1 1993/11/18 08:40:51 jrs Exp
+ *
+ * Thu Jul 14 07:32:40 1994: faith@cs.unc.edu added changed from Adam
+ * J. Richter (adam@adam.yggdrasil.com) so that /proc/filesystems is used
+ * if no -t option is given.  I modified his patches so that, if
+ * /proc/filesystems is not available, the behavior of mount is the same as
+ * it was previously.
+ *
+ * Wed Sep 14 22:43:00 1994: Mitchum DSouza
+ * (mitch@mrc-applied-psychology.cambridge.ac.uk) added support for mouting
+ * the "loop" device.
+ *
+ * Wed Sep 14 22:55:10 1994: Sander van Malssen (svm@kozmix.hacktic.nl)
+ * added support for remounting readonly file systems readonly.
+ *
  */
 
 #include "sundries.h"
 
 #include <linux/fs.h>
 
+int del_loop (const char *);
+
 /* True for fake mount (-f).  */
 int fake = 0;
 
@@ -188,6 +204,37 @@ fix_opts_string (int flags, char *extra_opts)
 
 /* Mount a single file system.  Return status,
    so don't exit on non-fatal errors.  */
+
+ 
+static int
+try_mount5 (char *spec, char *node, char **type, int flags, char *mount_opts) {
+   FILE *procfs_file;
+   char line[100];
+   char fsname[50];
+   
+   if (*type) return mount5 (spec, node, *type, flags & ~MS_NOSYS, mount_opts);
+   if (( procfs_file = fopen("/proc/filesystems", "r")) == NULL) {
+				/* If /proc/filesystems is not available,
+				   preserve the old behavior of mount. */
+      return mount5 (spec,
+		     node,
+		     FSTYPE_DEFAULT,
+		     flags & ~MS_NOSYS, mount_opts);
+   }
+   while (fgets(line, sizeof(line), procfs_file)) {
+      if (sscanf (line, "nodev %[^\n]\n", fsname) == 1) continue;
+      if (sscanf (line, " %[^ \n]\n", fsname) != 1) continue;
+      if (mount5 (spec, node, fsname, flags & ~MS_NOSYS, mount_opts) == 0) {
+      *type=xstrdup(fsname);
+      fclose(procfs_file);
+      return 0;
+      }
+   }
+   fclose(procfs_file);
+   return -1;
+}
+
+
 static int
 mount_one (char *spec, char *node, char *type, char *opts, int freq, int pass)
 {
@@ -196,13 +243,13 @@ mount_one (char *spec, char *node, char *type, char *opts, int freq, int pass)
   int flags;
   char *extra_opts;
   char *mount_opts;
+  int anti_recurse = 0;
+  int loop=0;
 
   if (type == NULL)
     {
       if (strchr (spec, ':') != NULL)
 	type = "nfs";
-      else
-	type = FSTYPE_DEFAULT;
     }
 
   parse_opts (xstrdup (opts), &flags, &extra_opts);
@@ -217,7 +264,20 @@ mount_one (char *spec, char *node, char *type, char *opts, int freq, int pass)
 
   mount_opts = extra_opts;
 
-  if (!fake && streq (type, "nfs"))
+  if (!fake && type && strncmp("lo@", type, 3)==0) {
+    extern int lomount (char *, char *, char *, char **, 
+			int *, char **, char **);
+    char *dev=type+3;
+
+    loop=1;
+    if (lomount (spec, node, dev, &type,
+		 &flags, &opts, &mount_opts) != 0)
+      return 1;
+    spec=dev;
+    mount_opts=NULL;
+  }
+
+  if (!fake && type && streq (type, "nfs"))
 #ifdef HAVE_NFS
     if (nfsmount (spec, node, &flags, &extra_opts, &mount_opts) != 0)
       return 1;
@@ -227,15 +287,17 @@ mount_one (char *spec, char *node, char *type, char *opts, int freq, int pass)
 
   block_signals (SIG_BLOCK);
 
-  if (fake || (mount5 (spec, node, type, flags & ~MS_NOSYS, mount_opts)) == 0)
+  if (fake
+      || (try_mount5 (spec, node, &type, flags & ~MS_NOSYS, mount_opts)) == 0)
     /* Mount succeeded, write mtab entry.  */
     {
       if (!nomtab)
 	{
 	  mnt.mnt_fsname = canonicalize (spec);
 	  mnt.mnt_dir = canonicalize (node);
-	  mnt.mnt_type = type;
-	  mnt.mnt_opts = fix_opts_string (flags & ~MS_NOMTAB, extra_opts);
+	  mnt.mnt_type = loop?"loop":type;
+	  mnt.mnt_opts = fix_opts_string (flags & ~MS_NOMTAB, 
+					  loop?opts:extra_opts);
 	  mnt.mnt_freq = freq;
 	  mnt.mnt_passno = pass;
       
@@ -260,6 +322,9 @@ mount_one (char *spec, char *node, char *type, char *opts, int freq, int pass)
       return 0;
     }
 
+  if (loop)
+	del_loop(spec);
+
   mnt_err = errno; /* work around for errno bug in sigprocmask */
 
   block_signals (SIG_UNBLOCK);
@@ -293,8 +358,26 @@ mount_one (char *spec, char *node, char *type, char *opts, int freq, int pass)
       error ("mount: %s is not a block device", spec); break;
     case ENXIO:
       error ("mount: %s is not a valid block device", spec); break;
-    case EACCES:
-      error ("mount: block device %s is not permitted on its filesystem", spec);
+    case EACCES:  /* pre-linux 1.1.38 */
+    case EROFS:   /* linux 1.1.38 and later */
+      if (anti_recurse)
+        {
+          error ("mount: block device %s is not permitted on its filesystem", spec);
+          break;
+        }
+      else
+        {
+         anti_recurse++;
+         if (opts)
+           {
+             opts = realloc(xstrdup(opts), strlen(opts)+3);
+             strcat(opts, ",ro");
+           }
+         else
+           opts = "ro";
+          error ("mount: block device %s is write-protected, mounting read-only", spec);
+          return mount_one (spec, node, type, opts, freq, pass);
+        }
       break;
     default:
       error ("mount: %s", strerror (mnt_err)); break;
