@@ -1,10 +1,26 @@
 /*
  * A umount(8) for Linux 0.99.
  * umount.c,v 1.1.1.1 1993/11/18 08:40:51 jrs Exp
+ *
+ * Wed Sep 14 22:43:54 1994: Sebastian Lederer
+ * (lederer@next-pc.informatik.uni-bonn.de) added support for sending an
+ * unmount RPC call to the server when an NFS-filesystem is unmounted.
  */
 
 #include "sundries.h"
 
+#ifdef HAVE_NFS
+#include <sys/socket.h>
+#include <sys/time.h>
+#include <netdb.h>
+#include <rpc/rpc.h>
+#include <rpc/pmap_clnt.h>
+#include <rpc/pmap_prot.h>
+#include "mount.h"
+#include <arpa/inet.h>
+#endif
+
+
 #ifdef notyet
 /* Nonzero for force umount (-f).  This needs kernel support we don't have.  */
 int force = 0;
@@ -16,6 +32,15 @@ int verbose = 0;
 /* True if ruid != euid.  */
 int suid = 0;
 
+#ifdef HAVE_NFS
+static int xdr_dir(XDR *xdrsp, char *dirp)
+{
+      return (xdr_string(xdrsp, &dirp, MNTPATHLEN));
+}
+#endif
+
+
+
 /* Umount a single device.  Return a status code, so don't exit
    on a non-fatal error.  We lock/unlock around each umount.  */
 static int
@@ -23,6 +48,21 @@ umount_one (const char *spec, const char *node)
 {
   int umnt_err;
   int isroot;
+  struct mntent *mnt;
+
+#ifdef HAVE_NFS
+      char buffer[256];
+      register CLIENT *clp;
+      struct sockaddr_in saddr;
+      struct timeval pertry, try;
+      enum clnt_stat clnt_stat;
+      int so = RPC_ANYSOCK;
+      char *p;
+      struct hostent *hostp;
+      char hostname[MAXHOSTNAMELEN];
+      char dirname[1024];
+#endif /* HAVE_NFS */
+  
 
   /* Special case for root.  As of 0.99pl10 we can (almost) unmount root;
      the kernel will remount it readonly so that we can carry on running
@@ -32,6 +72,62 @@ umount_one (const char *spec, const char *node)
      much.  [sct May 29, 1993] */
   isroot = (streq (node, "/") || streq (node, "root"));
 
+#ifdef HAVE_NFS
+      strcpy(buffer,spec);
+              /* spec is constant so must use own buffer */
+      if((p=strchr(buffer,':')))
+      {
+              *p='\0';
+              strcpy(hostname,buffer);
+              strcpy(dirname,p+1);
+#ifdef DEBUG
+              printf("host: %s, directory: %s\n", hostname,dirname);
+#endif
+
+
+              if (hostname[0] >= '0' && hostname[0] <= '9')
+              {
+                      saddr.sin_addr.s_addr = inet_addr(hostname);
+              }
+              else
+              if ((hostp = gethostbyname(hostname)) == NULL)
+              {
+                      fprintf(stderr, "mount: can't get address for %s\n", hostname);
+                      return(1);
+              }
+              else
+              {
+                              memcpy(&saddr.sin_addr, hostp->h_addr, hostp->h_length);
+              }
+
+              saddr.sin_family = AF_INET;
+              saddr.sin_port = 0;
+              pertry.tv_sec = 3;
+              pertry.tv_usec = 0;
+              if ((clp = clntudp_create(&saddr, MOUNTPROG, MOUNTVERS,
+                          pertry, &so)) == NULL)
+                      {
+                              clnt_pcreateerror("Cannot MOUNTPROG PRC");
+                              return (1);
+                      }
+              clp->cl_auth = authunix_create_default();
+              try.tv_sec = 20;
+              try.tv_usec = 0;
+              clnt_stat = clnt_call(clp, MOUNTPROC_UMNT,
+                              xdr_dir, dirname,
+                              xdr_void, (caddr_t)0,
+                              try);
+
+              if (clnt_stat != RPC_SUCCESS)
+              {
+                      clnt_perror(clp, "Bad UMNT RPC");
+                      return (1);
+              }
+              auth_destroy(clp->cl_auth);
+              clnt_destroy(clp);
+      }
+#endif /* HAVE_NFS */
+ 
   if (!isroot)
 	  lock_mtab ();
 
@@ -40,15 +136,30 @@ umount_one (const char *spec, const char *node)
     {
       if (verbose)
 	printf ("%s umounted\n", spec);
-      
+
       if (!isroot)
 	{
+				/* Special stuff for loop devices */
+	  open_mtab("r");
+	  if ((mnt = getmntfile (spec)) ||
+	      (mnt = getmntfile (node))) {
+	     if (mnt && streq(mnt->mnt_type, "loop")) {
+		extern int del_loop(const char *);
+		 
+		if (del_loop(spec))
+		      goto fail;
+	     }
+	  }
+	  close_mtab();
+
+				/* Non-loop stuff */
 	  update_mtab (node, NULL);
 	  unlock_mtab ();
 	}
       return 0;
     }
 
+fail:
   /* Umount failed, complain, but don't die.  */
   umnt_err = errno;
   if (!isroot)
